local dewdrop = AceLibrary("Dewdrop-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Click2Cast")
local T = AceLibrary("Tablet-2.0")

local actions = {
	menu = "Menu",
	spell = "Spell",
	target = "Target",
	focus = "Focus",
	assist = "Assist",
	macro = "Macro",
	none = "None",
}
local keybindingOnly = {BUTTON1=true, BUTTON2=true, BUTTON3=true, BUTTON4=true, BUTTON5=true}

-- Iterator for outputting correct keybind order
local function bindSortClosure()
    local modIndex = {
 		[""]        = 1,
 		["ALT-"]    = 2,
		["CTRL-"]   = 3,
		["SHIFT-"]  = 4,        
    }
    
	return function (a, b)
        local aButton, bButton = a:match("(%d+)"), b:match("(%d+)")
        local aButtonMod, bButtonMod = a:gsub("BUTTON%d", ""),b:gsub("BUTTON%d", "")
        
        if aButton == bButton then
            return (modIndex[aButtonMod] or 99) < (modIndex[bButtonMod] or 99)
        else
            return ((aButton or 99) < (bButton or 99))
        end
	end
end

local function EachBind(t)
	local i,sortedBinds = 0, {} 

	for k in pairs(t) do
		table.insert(sortedBinds, k)
	end

    table.sort(sortedBinds, bindSortClosure())

	return function()
		i = i + 1
		return sortedBinds[i], t[sortedBinds[i]]
	end
end

local function createSubMenu(self, value, order)
	local name = dewdrop:EncodeKeybinding(value)
	self.optionsTable.args[value] = {
		name = name or " ",
		desc = name or " ",
        order = order,
		type = 'group',
		args = {
			actions = {
				type = 'text',
				name = L["Set Action"],
				desc = L["actions"],
				order = 1,
				validate = actions,
				get = function() 
					return self.db.profile.clickOptions[value].action
				end,
				set = function(option)
					self.db.profile.clickOptions[value].action = option
					self:EnableFrames()
                    if self.actionText[self.db.profile.clickOptions[value].action] then
                        self.optionsTable.args[value].args.action.name = L[option]
                        self.optionsTable.args[value].args.action.desc = L[option.."Desc"]
                        dewdrop:Refresh(2)
                    end
				end,
			},
			remove = {
				type = 'execute',
				name = L["Remove"],
				desc = L["RemoveDesc"],
				func = function()
					self.db.profile.clickOptions[value] = nil
					self.optionsTable.args[value] = nil
					self:EnableFrames()
					dewdrop:Refresh(1)
					dewdrop:Close(2)
				end,
			},
			action = {
				name = L["Action Text"],
				type = 'text',
				desc = L["actiontext"],
                order = 2,
				get = function()
					return self.db.profile.clickOptions[value].actiontext or ""
				end,
				usage = "",
				set = function(option)
					self.db.profile.clickOptions[value].actiontext = option
					self:EnableFrames()
				end,
				hidden = function()
					return not self.actionText[self.db.profile.clickOptions[value].action]
				end,
			}
		}
	}
    if self.actionText[self.db.profile.clickOptions[value].action] then
        self.optionsTable.args[value].args.action.name = L[self.db.profile.clickOptions[value].action]
        self.optionsTable.args[value].args.action.desc = L[self.db.profile.clickOptions[value].action.."Desc"]
    end
end

function Click2Cast:CreateMenu()
	self.optionsTable = {
		handler = Click2Cast,
		type = 'group',
		args = {},
	}
	
	self.optionsTable.args.new = {
		name = "New binding",
		desc = "New binding",
		order = 200,
		type = 'text',
		validate = "keybinding",
		set = function(value)
			if not self.optionsTable.args[value] then
				createSubMenu(self, value)
			end
		end,
		get = function()
			return nil
		end,
		keybindingOnly = keybindingOnly,
	}
    local order = 1
	for k in EachBind(self.db.profile.clickOptions) do
        createSubMenu(self, k, order)
        order = order + 1
	end
	
	Click2Cast:RegisterChatCommand({ "/Click2Cast", "/c2c", "/cc" }, self.optionsTable)
	Click2Cast.OnMenuRequest = self.optionsTable
end

function Click2Cast:OnTooltipUpdate()
    local cat
    
    local showAction = {["spell"] = true,}
    local button
    for k,v in EachBind(self.db.profile.clickOptions) do
        if button and button:match("(%d+)") ~= k:match("(%d+)") or not button then
            cat = T:AddCategory(
                'text', dewdrop:EncodeKeybinding(k),
                'columns', 2,
                'child_textR', 1,
                'child_textG', 1,
                'child_textB', 0,
                'child_textR2', 1,
                'child_textG2', 1,
                'child_textB2', 1
            )
        end
        local noMod
        if k:gsub("-BUTTON%d", "") == k then
            noMod = L["No Modifier"]
        end
        if v.actiontext and showAction[v.action] then
            cat:AddLine(
                'text', noMod or dewdrop:EncodeKeybinding(k),
                'text2', v.actiontext
            )        
        else
            cat:AddLine(
                'text', noMod or dewdrop:EncodeKeybinding(k),
                'text2', L[v.action]
            )
        end
        button = k:match("(%d+)")
    end
    
    T:SetHint(L["Right-click to configure clickcast actions."])
end

function Click2Cast:RecreateMenu()
	for k in pairs(self.optionsTable.args) do
		if k:find("^ALT.-BUTTON%d$") or k:find("^CTRL.-BUTTON%d$") or k:find("^SHIFT.-BUTTON%d$") or k:find("^BUTTON%d$") then
			self.optionsTable.args[k] = nil
		end
	end
	for k in EachBind(self.db.profile.clickOptions) do
		createSubMenu(self, k)
	end
end
